/* Copyright 2013 Forex Capital Markets LLC

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use these files except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Globalization;

using Candleworks.PriceHistoryMgr;
using Candleworks.QuotesMgr;
using fxcore2;

namespace GetHistPricesGUI
{
    /// <summary>
    /// GetHistory activity.
    /// </summary>
    class GetHistoryActivity : IWaitableActivity
    {
        private bool mSuccess;
        private string mErrorMessage;
        private IWaitableActivityCallback mCallback;
        private GetHistoryParams mGetHistoryParams;
        private PriceAPIController mPriceAPIController;
        private string mFileName;

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="getHistoryParams">The GetHistory parameters.</param>
        /// <param name="controller">The price controller.</param>
        /// <param name="filename">The output filename.</param>
        public GetHistoryActivity(GetHistoryParams getHistoryParams, PriceAPIController controller, string filename)
        {
            mGetHistoryParams = getHistoryParams;
            mPriceAPIController = controller;
            mFileName = filename;
        }

        #region IWaitableActivity methods

        public string Name
        {
            get
            {
                return Properties.Resources.GetHistoryActivityName;
            }
        }

        public bool Success
        {
            get
            {
                return mSuccess;
            }
        }

        public string ErrorMessage
        {
            get
            {
                return mErrorMessage;
            }
        }

        public bool StartActivity(IWaitableActivityCallback callback)
        {
            mCallback = callback;

            try
            {
                // create timeframe
                IPriceHistoryCommunicator communicator = mPriceAPIController.PriceHistoryCommunicator;
                if (communicator == null)
                {
                    mErrorMessage = Properties.Resources.CommunicatorIsNotReadyError;
                    return false;
                }
                O2GTimeframe timeframe = communicator.TimeframeFactory.create(mGetHistoryParams.Period);

                mPriceAPIController.OnCollectionLoaded += new PriceAPIController_CollectionLoaded(PriceAPIController_CollectionLoaded);
                mPriceAPIController.OnErrorEvent += new PriceAPIController_Error(PriceAPIController_Error);

                // set required WeekendData mode
                WeekendCandleMode mode = WeekendCandleMode.WeekendCandleShow;
                switch (mGetHistoryParams.WeekendDataMode)
                { 
                    case GetHistoryParams.WeekendData.Hide:
                        mode = WeekendCandleMode.WeekendCandleHide;
                        break;
                    case GetHistoryParams.WeekendData.MergeIntoMonday:
                        mode = WeekendCandleMode.WeekendCandleMergeToNextWeek;
                        break;
                    case GetHistoryParams.WeekendData.Show:
                        mode = WeekendCandleMode.WeekendCandleShow;
                        break;
                }
                communicator.getQuotesManager().mode = mode;

                mPriceAPIController.RequestHistory(mGetHistoryParams.Symbol, timeframe,
                    mGetHistoryParams.FromDate, mGetHistoryParams.ToDate, mGetHistoryParams.Count);

                return true;
            }
            catch (Exception e)
            {
                mPriceAPIController.OnCollectionLoaded -= new PriceAPIController_CollectionLoaded(PriceAPIController_CollectionLoaded);
                mPriceAPIController.OnErrorEvent -= new PriceAPIController_Error(PriceAPIController_Error);

                mErrorMessage = e.Message;
                return false;
            }
        }

        #endregion

        #region Event handlers

        private void PriceAPIController_Error(string error)
        {
            mSuccess = false;
            mErrorMessage = error;

            mPriceAPIController.OnCollectionLoaded -= new PriceAPIController_CollectionLoaded(PriceAPIController_CollectionLoaded);
            mPriceAPIController.OnErrorEvent -= new PriceAPIController_Error(PriceAPIController_Error);

            mCallback.OnActivityCompleted();
        }

        private void PriceAPIController_CollectionLoaded(IPeriodCollection collection)
        {
            mSuccess = true;

            mPriceAPIController.OnCollectionLoaded -= new PriceAPIController_CollectionLoaded(PriceAPIController_CollectionLoaded);
            mPriceAPIController.OnErrorEvent -= new PriceAPIController_Error(PriceAPIController_Error);

            try
            {
                SaveCollection(collection, mFileName);
            }
            catch (Exception e)
            {
                mErrorMessage = e.Message;
                mSuccess = false;
            }

            mCallback.OnActivityCompleted();
        }

        #endregion

        /// <summary>
        /// Saves the price history collection to a CSV file.
        /// </summary>
        /// <param name="collection">The collection to save.</param>
        /// <param name="filename">The output filename.</param>
        /// <exception cref="Exception">If some error occurred.</exception>
        private void SaveCollection(IPeriodCollection collection, string filename)
        {
            CultureInfo cultureInfo = CultureInfo.CurrentCulture;
            string separator = cultureInfo.TextInfo.ListSeparator;

            // get symbol precision
            int precision = 5;
            Instruments instruments = mPriceAPIController.QuotesManager.getInstruments();
            Instrument instrument = instruments.find(mGetHistoryParams.Symbol);
            if (instrument != null)
                precision = instrument.getPrecision();
            string doubleFormat = "F" + precision;

            using (StreamWriter file = new StreamWriter(filename, false))
            {
                string header = "DateTime" + separator +
                                "BidOpen"  + separator + "BidHigh"  + separator +
                                "BidLow"   + separator + "BidClose" + separator +
                                "AskOpen"  + separator + "AskHigh"  + separator +
                                "AskLow"   + separator + "AskClose" + separator +
                                "Volume";
                file.WriteLine(header);

                foreach (IPeriod period in collection)
                {
                    string line = period.Time.ToString(cultureInfo)                    + separator +
                                  period.Bid.Open.ToString(doubleFormat, cultureInfo)  + separator +
                                  period.Bid.High.ToString(doubleFormat, cultureInfo)  + separator +
                                  period.Bid.Low.ToString(doubleFormat, cultureInfo)   + separator +
                                  period.Bid.Close.ToString(doubleFormat, cultureInfo) + separator +
                                  period.Ask.Open.ToString(doubleFormat, cultureInfo)  + separator +
                                  period.Ask.High.ToString(doubleFormat, cultureInfo)  + separator +
                                  period.Ask.Low.ToString(doubleFormat, cultureInfo)   + separator +
                                  period.Ask.Close.ToString(doubleFormat, cultureInfo) + separator +
                                  period.Volume.ToString(cultureInfo);
                    file.WriteLine(line);
                }
            }
        }
    }
}
