﻿/* Copyright 2013 Forex Capital Markets LLC

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use these files except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace GetHistPricesGUI
{
    /// <summary>
    /// The form that allows to get historical prices and save them to a CSV file.
    /// </summary>
    partial class GetHistoryForm : Form
    {
        private GetHistoryParams mModel;
        private PriceAPIController mPriceAPIController;

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="controller">The prices controller.</param>
        public GetHistoryForm(PriceAPIController controller)
        {
            InitializeComponent();

            CultureInfo cultureInfo = CultureInfo.CurrentCulture;
            string dateTimeFormat = cultureInfo.DateTimeFormat.ShortDatePattern + " " + cultureInfo.DateTimeFormat.ShortTimePattern;
            dateTimePickerFrom.CustomFormat = dateTimeFormat;
            dateTimePickerTo.CustomFormat = dateTimeFormat;

            mPriceAPIController = controller;
            mPriceAPIController.OnStateChange += new PriceAPIController_StateChange(PriceAPIController_StateChange);

            FillSymbols();
            UpdateControlsState();
            UpdateSessionStatus();

            mModel = new GetHistoryParams();
            mModel.Load();
            UpdateView();
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void comboBoxSymbol_TextChanged(object sender, EventArgs e)
        {
            UpdateControlsState();
        }

        private void comboBoxPeriod_TextChanged(object sender, EventArgs e)
        {
            UpdateControlsState();
        }

        private void checkBoxFrom_CheckedChanged(object sender, EventArgs e)
        {
            UpdateControlsState();
        }

        private void checkBoxTo_CheckedChanged(object sender, EventArgs e)
        {
            UpdateControlsState();
        }

        private void checkBoxCount_CheckedChanged(object sender, EventArgs e)
        {
            UpdateControlsState();
        }

        private void textBoxCount_TextChanged(object sender, EventArgs e)
        {
            UpdateControlsState();
        }

        private void buttonGetHistory_Click(object sender, EventArgs e)
        {
            UpdateModel();
            mModel.Save();

            // prompt for the output filename
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.OverwritePrompt = true;
            dlg.Filter = Properties.Resources.SaveHistoryFilter;
            dlg.FileName = CreateDefaultFileName();
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                // start loading prices in background
                GetHistoryActivity activity = new GetHistoryActivity(mModel, mPriceAPIController, dlg.FileName);
                WaitForm form = new WaitForm(activity);
                form.ShowDialog();
                if (activity.Success)
                    Util.ShowMesasge(Properties.Resources.HistorySavedMessage);
                else
                    Util.ShowError(activity.ErrorMessage);
            }
        }

        private static string MakeValidFileName(string name)
        {
            string invalidChars = System.Text.RegularExpressions.Regex.Escape(new string(System.IO.Path.GetInvalidFileNameChars()));
            string invalidRegStr = string.Format(@"([{0}]*\.+$)|([{0}]+)", invalidChars);

            return System.Text.RegularExpressions.Regex.Replace(name, invalidRegStr, "");
        }

        /// <summary>
        /// Creates the default filename for a history CSV file.
        /// </summary>
        /// <returns>The default name.</returns>
        private string CreateDefaultFileName()
        {
            string res = mModel.Symbol + " " + mModel.Period;

            string fromDate = "";
            if (mModel.FromDate != Candleworks.PriceHistoryMgr.Constants.ZERODATE)
                fromDate = "From_" + mModel.FromDate.ToString("MM-dd-yyyy_hhmm");

            string toDate = "";
            if (mModel.ToDate != Candleworks.PriceHistoryMgr.Constants.ZERODATE)
                toDate = "To_" + mModel.ToDate.ToString("MM-dd-yyyy_hhmm");

            string count = "";
            if (mModel.Count > 0)
                count = "Count_" + mModel.Count.ToString();

            if (fromDate.Length > 0 ||
                toDate.Length > 0 ||
                count.Length > 0)
                res += " (";
            if (fromDate.Length > 0)
                res += fromDate;
            if (toDate.Length > 0)
            {
                if (fromDate.Length > 0)
                    res += " ";
                res += toDate;
            }
            if (count.Length > 0)
            {
                if (fromDate.Length > 0 || toDate.Length > 0)
                    res += " ";
                res += count;
            }
            if (fromDate.Length > 0 ||
                toDate.Length > 0 ||
                count.Length > 0)
                res += ")";

            return MakeValidFileName(res);
        }

        /// <summary>
        /// Updates state of controls after some changes.
        /// </summary>
        private void UpdateControlsState()
        {
            string symbol = comboBoxSymbol.Text;
            string period = comboBoxPeriod.Text;
            buttonGetHistory.Enabled = symbol.Length > 0 && period.Length > 0 &&
                mPriceAPIController.IsReady;

            dateTimePickerFrom.Enabled = checkBoxFrom.Checked;
            dateTimePickerTo.Enabled = checkBoxTo.Checked;
            textBoxCount.Enabled = checkBoxCount.Checked;
        }

        /// <summary>
        /// Fills Symbols combobox by Offers available in the trading session.
        /// </summary>
        private void FillSymbols()
        {
            comboBoxSymbol.Items.Clear();

            IOfferCollection offers = mPriceAPIController.Offers;
            foreach (IOffer offer in offers)
                comboBoxSymbol.Items.Add(offer.Instrument);
        }

        /// <summary>
        /// Update the view by the model.
        /// </summary>
        private void UpdateView()
        {
            comboBoxSymbol.Text = mModel.Symbol;
            comboBoxPeriod.Text = mModel.Period;
            comboBoxWeekendData.SelectedIndex = (int)mModel.WeekendDataMode;

            checkBoxFrom.Checked = mModel.FromDate != Candleworks.PriceHistoryMgr.Constants.ZERODATE;
            if (checkBoxFrom.Checked)
                dateTimePickerFrom.Value = mModel.FromDate;

            checkBoxTo.Checked = mModel.ToDate != Candleworks.PriceHistoryMgr.Constants.ZERODATE;
            if (checkBoxTo.Checked)
                dateTimePickerTo.Value = mModel.ToDate;

            checkBoxCount.Checked = mModel.Count > 0;
            if (checkBoxCount.Checked)
                textBoxCount.Text = mModel.Count.ToString();
        }

        /// <summary>
        /// Update the model by the view.
        /// </summary>
        private void UpdateModel()
        {
            mModel.Symbol = comboBoxSymbol.Text;
            mModel.Period = comboBoxPeriod.Text;
            mModel.WeekendDataMode = (GetHistoryParams.WeekendData)comboBoxWeekendData.SelectedIndex;

            mModel.FromDate = checkBoxFrom.Checked ? dateTimePickerFrom.Value : Candleworks.PriceHistoryMgr.Constants.ZERODATE;
            mModel.ToDate = checkBoxTo.Checked ? dateTimePickerTo.Value : Candleworks.PriceHistoryMgr.Constants.ZERODATE;
            mModel.Count = 0;
            if (checkBoxCount.Checked)
                Int32.TryParse(textBoxCount.Text, out mModel.Count);
        }

        private void PriceAPIController_StateChange(bool isReady)
        {
            if (InvokeRequired)
                Invoke(new MethodInvoker(UpdateSessionStatus));
            else
                UpdateSessionStatus();
        }

        /// <summary>
        /// Updates Session Status label.
        /// </summary>
        private void UpdateSessionStatus()
        {
            labelStatus.Text = mPriceAPIController.IsReady ? Properties.Resources.ReadyStatus :
                Properties.Resources.NotReadyStatus;
            UpdateControlsState(); // update Get History button state
        }
    }
}
